#include <QtUiTools/QUiLoader>
#include <QtCore/QStringList>
#include <QtCore/QFile>
#include <QtGui/QApplication>
#include <QtGui/QWidget>

#include <Qt3Support/Q3HBox>
#include <Qt3Support/Q3VBox>


#include "uiutils.h"

void UiUtils::install( QScriptEngine *engine )
{
    installWidgets( engine );
    installBoxWidgets( engine );

    QScriptValue utilsObject = engine->newObject();

    QScriptValue fun = engine->scriptValue( UiUtils::load );
    utilsObject.setProperty("load", fun);
    fun = engine->scriptValue( UiUtils::exec );
    utilsObject.setProperty("exec", fun);

    QScriptValue globalObject = engine->globalObject();
    globalObject.setProperty( "UiUtils", utilsObject );

}

void UiUtils::installWidgets( QScriptEngine *engine )
{
    QScriptValue globalObject = engine->globalObject();
    QUiLoader loader;

    QStringList widgets = loader.availableWidgets();
    for ( int i=0; i < widgets.size(); ++i ) {
	QScriptValue fun = engine->scriptValue( createWidget );
	fun.setProperty( QString("functionName"), engine->scriptValue(widgets[i]) );

	globalObject.setProperty(widgets[i], fun);
    }
}

QScriptValue UiUtils::createWidget(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() > 1 )
	return context->throwError("Load takes one argument");

    QWidget *parent = 0;
    if ( context->argumentCount() ) {
	parent = qscript_cast<QWidget*>(context->argument(0));

	if ( !parent )
	    return context->throwError("The argument to load should be a QWidget");
    }

    QString self = context->callee().property( "functionName" ).toString();
    QUiLoader loader;
    QWidget *w = loader.createWidget( self, parent );
    return engine->scriptValueFromQObject( w );
}

void UiUtils::installBoxWidgets( QScriptEngine *engine )
{
    QScriptValue fun;
    QScriptValue globalObject = engine->globalObject();

    fun = engine->scriptValue( UiUtils::createBox );
    fun.setProperty( QString("functionName"), engine->scriptValue("QHBox") );
    globalObject.setProperty("QHBox", fun);

    fun = engine->scriptValue( UiUtils::createBox );
    fun.setProperty( QString("functionName"), engine->scriptValue("QVBox") );
    globalObject.setProperty("QVBox", fun);
}

QScriptValue UiUtils::createBox(QScriptContext *context, QScriptEngine *engine)
{
    QWidget *parent = 0;
    if ( context->argumentCount() ) {
	parent = qscript_cast<QWidget*>(context->argument(0));

	if ( !parent )
	    return context->throwError("The parent should be a QWidget");
    }

    QString self = context->callee().property( "functionName" ).toString();

    QWidget *w;
    if ( self == QString("QHBox") ) {
	w = new Q3HBox(parent);
    }
    else if ( self == QString("QVBox") ) {
	w = new Q3VBox(parent);
    }

    return engine->scriptValueFromQObject( w );
}

QScriptValue UiUtils::load(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() != 1 )
	return context->throwError("Load takes one argument");

    QUiLoader loader;
    QString filename = context->argument(0).toString();
    QFile f( filename );
    if ( !f.open(QIODevice::ReadOnly) )
	return context->throwError(QString("Unable to open '%1'").arg(filename) );

    QWidget *w = loader.load( &f );

    return engine->scriptValueFromQObject( w );
}

QScriptValue UiUtils::exec(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() )
	return context->throwError("Load takes no arguments");

    return engine->scriptValue( QApplication::exec() );
}
